#!/usr/bin/env python

# -*- coding: utf-8 -*-

# std import
import sys
import argparse
import os.path
from collections import defaultdict
import csv
import re
import math


class Variant(object):

    def __init__(self, type, comment):
        self.type = type
        self.comment = comment

    def __eq__(self, other):
        if self.type != other.type:
            return False

        if self.comment != other.comment:
            return False

        return True

    def __str__(self):
        return "%s_%s" % (str(self.type), str(self.comment))

    def __repr__(self):
        return "<%s - %s>" % (self.type, self.comment)

    def __hash__(self):
        return hash(self.type + self.comment)


def main():
    """ The main function of vde no argument """

    enable_input = [name.split("2")[0] for name in globals().keys()
                    if name.endswith("2eva")]

    parser = argparse.ArgumentParser(
        prog="vde",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("-e", "--experiment",
                        type=str,
                        help="File of experimente result.",
                        required=True)
    parser.add_argument("-t", "--truth",
                        type=str,
                        help="File of truth result.",
                        required=True)
    parser.add_argument("-d", "--delta",
                        type=int,
                        help="Acceptable diff betwen truth and experimente.",
                        default=5)
    parser.add_argument("-ef", "--experiment-format",
                        type=str,
                        help="Format of experiment file",
                        choices=enable_input,
                        default="eva")
    parser.add_argument("-tf", "--truth-format",
                        type=str,
                        help="Format of truth file",
                        choices=enable_input,
                        default="eva")

    # parsing cli argument
    argument = vars(parser.parse_args())

    expfunc = globals()[argument["experiment_format"]+"2eva"]
    truthfunc = globals()[argument["truth_format"]+"2eva"]

    experiment, count = expfunc(argument["experiment"])
    truth, count = truthfunc(argument["truth"])

    result = compare(experiment, truth, argument["delta"])

    result_printing(result, count)


def result_printing(result, count):
    """ Printing the result in csv format """

    head = ",".join(("type", "TP", "FP", "recall", "precision"))
    print(head)
    for gap in result.keys(): 
        total = result[gap]["TP"] + result[gap]["FP"]

        prec = 1 if total == 0 else result[gap]["TP"]/float(total)
        recall = 1 if count[gap] == 0 else result[gap]["TP"]/float(count[gap])

        print(",".join((str(gap),
                        str(result[gap]["TP"]),
                        str(result[gap]["FP"]),
                        str(recall),
                        str(prec))))


def compare(exp, truth, delta):
    """ Compare experimente and truth return TP FP precision and recall
    for each type """

    result = defaultdict(lambda: defaultdict(int))
    
    exp_pos = set(exp.keys())
    tru_pos = set(truth.keys())

    for exact_pos in set(exp_pos & tru_pos):
        for variant in exp[exact_pos]:
            if variant in truth[exact_pos]:
                result[variant.type]["TP"] += 1
            else: 
                result[variant.type]["FP"] += 1

    not_found = set(exp_pos - (exp_pos & tru_pos))
    for fuzzy_pos in set(exp_pos - (exp_pos & tru_pos)):
        end = False
	for pos in range(fuzzy_pos - delta, fuzzy_pos + delta + 1):
            for variant in exp[fuzzy_pos]:
                if variant.type in ("snp", "multi_snp"):
		    result[variant.type]["FP"] += 1
                    try:
                        not_found.remove(fuzzy_pos)
                    except KeyError:
                        pass
                    end = True
                    break

                if variant in truth[pos]:
                    result[variant.type]["TP"] += 1
                    try:
                        not_found.remove(fuzzy_pos)
                    except KeyError:
                        pass
                    end = True
                    break
            if end :
                break

    for pos in not_found:
        for variant in set(exp[pos]):
            result[variant.type]["FP"] += 1

    return result


def eva2eva(filename):
    """ Read eva file and return value in dict
    position is key and type is value """

    __check_file_exist(filename)

    data = defaultdict(list)
    count = defaultdict(int)

    with open(filename) as csvfile:
        linereader = csv.reader(csvfile)
        for val in linereader:
            data[int(val[0])].append(Variant(val[1], val[2]))
            count[val[1]] += 1

    return data, count


def breakpoints2eva(filename):
    """ Read breakpoint file and return value in dict
    position is key  and type is value """

    __check_file_exist(filename)

    data = defaultdict(list)
    count = defaultdict(int)

    mtg2eva = {"HOM": "homo",
               "HET": "hete",
               "SNP": "snp",
               "MSNP": "multi_snp",
               "DEL": "deletion",
               "BACKUP": "backup"}

    findpos = re.compile(r'pos_([-\d]+)')
    findtype = re.compile(r'_([a-zA-Z]+)$')
    findcomment = re.compile(r'contig_\d+_(.+)_pos')

    with open(filename) as filehand:
        for line in filehand:
            line = line.strip()
            if line.startswith(">left_contig_"):
		data[int(findpos.search(line).group(1))].append(Variant(
                    mtg2eva[findtype.search(line).group(1)],
                    findcomment.search(line).group(1)))

                count[mtg2eva[findtype.search(line).group(1)]] += 1

    return data, count


def __add_in_data_count(pos, type_gap, data, counter):
    """ Add value pos: type_gap in data and increment counter[data] """

    data[pos].add(type_gap)
    counter[type_gap] += 1


def __check_file_exist(filename):
    """ If file doesn't exist trow assert """

    assert os.path.isfile(filename), "Error when I try open " + filename


if(__name__ == '__main__'):
    main()



import sys
import csv
from Bio.Seq import Seq
from Bio import SeqIO
from collections import defaultdict
import getopt
csv.field_size_limit(100000000)


def main():
    print(sys.argv[1:])
    try:
        opts, args = getopt.getopt(sys.argv[1:], "s:g:o:", ["snp=", "genome=", "genome_altered="])
    except getopt.GetoptError:
        # print help information and exit:
		#print ('error')  # will print something like "option -a not recognized"
        sys.exit(2)
    print('start')
    genome_parser=""
    vcf_reader=""
    out_m=csv.writer(open(sys.argv[3],"w"),delimiter="\n")
    dic_snp={}
	
    for opt, arg in opts:
        print(opt, arg)
        if opt in ('-s', "--snp"):
            vcf_reader = arg
	#print(i)
        elif opt in ('-g', "--genome"):
            genome_parser = arg
	#print(r)
        elif opt in ('-o', "--genome_altered"):
            out_m = arg
        else:
	        assert False, "unhandled option"
    print('start')
    
    dic_snp=extract_snp(vcf_reader)	
    alter_genome(genome_parser,dic_snp,out_m)
	
def is_valid(inser) :
	allowed="ATCGatcg"
	if all(c in allowed for c in inser ) :
		return True
	else : 
		return False
	
def insert_snp(genome,chromosome,dic_snp) :
	nb_error=0
	#print (chromosome)
	#print ("len before", len(genome))
	if chromosome in dic_snp :
		for listing in dic_snp[chromosome] :
			if listing[1]==genome[listing[0]-1] :
				genome[listing[0]-1]=listing[2]
			else :
				print( "Error SNP: in genome ",genome[listing[0]-2:listing[0]+2], "in vcf ", listing[1],"at position", listing[0])
				nb_error+=1
	print (" nb SNP substitution failed",nb_error)
	return genome

def extract_snp(vcf_reader):
    dic_snp={}
    vcf_readers=csv.reader(open(vcf_reader,'r'),delimiter='\t')
    count_snp=0
    for elits in vcf_readers :
        if '#' not in elits[0] and '@' not in elits[0] :
            if len(elits[3])==1 and len(elits[4])==1  and is_valid(elits[3])==True and is_valid(elits[4])==True:
                dic_snp.setdefault(elits[0],[]).append((int(elits[1]),elits[3],elits[4]))
                count_snp+=1
    print("total_snp in vcf : ", count_snp)
    return dic_snp
			
def alter_genome(genome_parser,dic_snp,out_m):
	genome_parsers=SeqIO.parse(genome_parser, "fasta")
	out_ms=csv.writer(open(out_m,"w"),delimiter="\n")
	for record in genome_parsers :
			elts=str(record.description)
			head=">" + str(record.description)
			old_sequence=list(str(record.seq).upper())
			old_length=len(old_sequence)
			sequence=""
			unmasked_seq=""
			num_exon=0
			total_length=0
			work_seq=insert_snp(old_sequence,elts,dic_snp)
			finals=''.join(work_seq)
			print( 'final', len(finals))
			out_m_lists=[head,finals]
			out_ms.writerow(out_m_lists)

if __name__ == "__main__":
    main()


from gatb import Graph
import csv
import sys
import numpy as np
import pandas as pd
from Bio.Seq import Seq
from Bio import SeqIO
import statistics
import pandas as pd
import getopt


def main():
    print(sys.argv[1:])
    try:
        opts, args = getopt.getopt(sys.argv[1:], "g:p:c:b:s:t:v:o:m:", ["graph=", "genome_parser=", "branching_outp=", "bkpt_file=", "truth_vcf=", "outp_context=","vcf_fill=","bkpt_outp=","threshold="])
    except getopt.GetoptError:
        # print help information and exit:
		#print ('error')  # will print something like "option -a not recognized"
        sys.exit(2)

    # Default parameters
	#print(opts)
    graph = ""
    genome_parser = ""
    branching_outp = ""
    bkpt_file = ""
    truth_vcf = ""
    outp_context = ""
    vcf_fill=""
    dic_parse = {}
    bkpt_outp=""
    threshold=0.80
    for opt, arg in opts:
        print(opt, arg)
        if opt in ('-g', "--graph"):
            graph = arg
	#print(i)
        elif opt in ('-p', "--genome_parser"):
            genome_parser = arg
	#print(r)
        elif opt in ('-c', "--branching_outp"):
            branching_outp = arg
	#print(i)
        elif opt in ('-b', "--bkpt_file"):
            bkpt_file = arg
        elif opt in ('-s', "--truth_vcf"):
            truth_vcf = arg
        elif opt in ('-t', "--outp_context"):
            outp_context = arg
        elif opt in ('-v', "--vcf_fill"):
            vcf_fill = arg
        elif opt in ('-o', "--bkpt_outp"):
            bkpt_outp = arg
        elif opt in ('-m', "--threshold"):
            threshold =int(arg)
        else:
	        assert False, "unhandled option"
    #dic_parse=parsing_genome_branching(graph, genome_parser)
    #parsing_genome_branching2(graph,genome_parser,branching_outp)
    #analyze_genomic_context(bkpt_file, branching_outp,bkpt_outp)
    #analyze_genomic_context2(bkpt_file,branching_outp,truth_vcf,vcf_fill)
    #write_context_genomic(dico_TP, dico_FP,outp_context)

    analyze_genomic_context_direct(bkpt_file, graph,genome_parser,bkpt_outp,threshold)
    
    
def analyze_genomic_context_direct(bkpt, graph_h5,genome,outp_bkpt,threshold):
    forma = "-in "+graph_h5
    graph = Graph(forma)
    graph
    genomes_parser = SeqIO.parse(open(genome), "fasta")
    genome_parser = SeqIO.parse(open(bkpt), "fasta")
    outp_find = csv.writer(open(outp_bkpt, 'w'), delimiter='\n')
    liste_good=[]
    liste_chrom_seen=[]
    liste_position = []
    dico_first={}
    dico_second={}
    total_bkpt=0
    count=0
    for element in genome_parser :
        if count%2==0 :
            dico_first.setdefault(element.description.split('_')[1], []).append(int(element.description.split('_')[3]))
            total_bkpt+=1
        count+=1
	
    for elt in genomes_parser :
        str_chromosome = str(elt.seq)
        id_chrom=str(elt.description)
        if id_chrom in dico_first :
            for value in dico_first[id_chrom] :
                 #print(value)
                 sum_degree=[]
                 for i in range (50) :
                    kmer = str_chromosome[value-i-31:value-i]
                    node = graph[kmer]
                    bytes(node)
                    assert node.reversed == node
                    sum_degree.append(node.out_degree)
                    sum_degree.append(node.in_degree)
                 percentage_concatenate = (sum_degree.count(1)+sum_degree.count(2))/(len(sum_degree))
                    #print(percentage_concatenate)
                 if percentage_concatenate > threshold :  
                    dico_second.setdefault(id_chrom, []).append(int(value))
    a=0
    for i in dico_second:
        a+=len(dico_second[i])
    print ("total breakpoints kept : ", a, " on ", total_bkpt)
    genome_parser = SeqIO.parse(open(bkpt), "fasta")
    for element in genome_parser :
        #print(element)
        if int(element.description.split('_')[3]) in dico_second[element.description.split('_')[1]] :
            outp_find.writerow([">"+element.description,element.seq])


def parsing_genome_branching(graph_h5, genome):
    forma = "-in "+graph_h5
    graph = Graph(forma)
    graph
    genome_parser = SeqIO.parse(open(genome), "fasta")
    dico_parse={}
    for chromosome in genome_parser:
        str_chromosome = str(chromosome.seq)
        #print(str_chromosome[0:31])
        for i in range(len(str_chromosome)-31):
            if "N" not in str_chromosome[i:i+31]:
                #print (str_chromosome[i:i+31])
                kmer = str_chromosome[i:i+31]
                node = graph[kmer]
                bytes(node)
                assert node.reversed == node
                position = i
                out_deg = node.out_degree
                in_deg = node.in_degree
                #if out_deg>1 or in_deg>1
                dico_parse.setdefault(chromosome.description, []).append((position, in_deg, out_deg))
            else:
                dico_parse.setdefault(chromosome.description, []).append((i, 0, 0))
    for a in dico_parse :
        print(a)
    return (dico_parse)


def parsing_genome_branching2(graph_h5, genome, outpt):
    forma = "-in "+graph_h5
    graph = Graph(forma)
    graph
    genome_parser = SeqIO.parse(open(genome), "fasta")
    output_bed = csv.writer(open(outpt, "w"), delimiter="\t")
    output_bed.writerow(["chr","position", "in_degree", "out_degree"])

    

    for chromosome in genome_parser:
        str_chromosome = str(chromosome.seq)
        #print(str_chromosome[0:31])
        for i in range(len(str_chromosome)-31):
            if "N" not in str_chromosome[i:i+31]:
                #print (str_chromosome[i:i+31])
                kmer = str_chromosome[i:i+31]
                node = graph[kmer]
                bytes(node)
                assert node.reversed == node
                position = i
                out_deg = node.out_degree
                in_deg = node.in_degree
                #if out_deg>1 or in_deg>1:
                output_bed.writerow([chromosome.description,position, in_deg, out_deg])
            else:
                output_bed.writerow([chromosome.description,i, 0, 0])


def analyze_genomic_context2(bkpt, branching_bed,truth_file,vcf_file):
    genome_parser = SeqIO.parse(open(bkpt), "fasta")
    input_bed = pd.read_csv(branching_bed, sep='\t')
    sum_FP = 0
    sum_TP = 0
    good_tp=0
    bad_tp=0
    good_fp=0
    bad_fp=0
    fp_remove=0
    fp_kept=0
    tp_kept=0
    outp_fill = csv.reader(open(vcf_file, 'r'), delimiter='\t')
    liste_fill=[]
    for elt in outp_fill :
        if "#" not in elt[0] and "@" not in elt[0] :
            liste_fill.append(int(elt[1]))
    #print(liste_fill)



    truth_parser = csv.reader(open(truth_file, 'r'), delimiter='\t')
    next(truth_parser, None)  # skip header
    liste_position = []
    liste_truth = []
    i=0
    for elements in truth_parser:
        if i % 2 == 0:
            liste_position.append(int(elements[2]))
            liste_truth.append(int(elements[2]))
        i += 1
    for element in genome_parser:
        #Avoid repetition from couple breakpoint
        if 'left' in str(element.description):
            #positon changes if it is a back up insertion
            if element.description.split('_')[2] == 'backup':
                pos = int(element.description.split('_')[4])
            else:
                pos = int(element.description.split('_')[3])
            test_in = input_bed[(input_bed['chr'] == element.description.split('_')[1]) & (input_bed['position'] > pos-100) & (input_bed['position'] <= pos-31)]
            liste_in=test_in.in_degree.tolist()
            liste_out=test_in.out_degree.tolist()
            percentage_concatenate = (liste_in.count(1)+liste_in.count(2)+liste_out.count(1)+liste_out.count(2))/(len(liste_in)+len(liste_out))
            if percentage_concatenate > 0.80:
                    sum_TP += 1
                   
                    if pos in liste_truth or pos-1 in liste_truth or pos+1 in liste_truth:
                        good_tp+=1
                        if pos in liste_fill or pos-1 in liste_fill or pos+1 in liste_fill:
                            #print('FP failed filled', element.description)
                            tp_kept += 1
                    else : 
                        bad_tp+=1
                        #print('FP failed', element.description,percentage_concatenate)
                        if pos in liste_fill or pos-1 in liste_fill or pos+1 in liste_fill:
                            #print('FP failed filled', element.description)
                            fp_kept += 1

                        

            else : 
                sum_FP+=1
                #print('FP',element.description, percentage_concatenate)
                if pos in liste_truth or pos-1 in liste_truth or pos+1 in liste_truth:
                    bad_fp += 1
                    #print('TP failed', element.description, percentage_concatenate)
                else : 
                    good_fp+=1
                    if pos in liste_fill or pos-1 in liste_fill or pos+1 in liste_fill:
                            #print('FP failed filled', element.description)
                            fp_remove+=1
    print("sum sup 0.5 ", sum_TP,"sum below 0.5 ", sum_FP)
    print('TP predicted ', good_tp,'TP_failed ',bad_tp,'FP predicted ', good_fp,'FP failed ',bad_fp, " fp removed ", fp_remove, "fp_kept",fp_kept,"tp_kept",tp_kept)

def analyze_genomic_context(bkpt, branching_bed,outp_bkpt):
    genome_parser = SeqIO.parse(open(bkpt), "fasta")
    input_bed = pd.read_csv(branching_bed, sep='\t')
    outp_find = csv.writer(open(outp_bkpt, 'w'), delimiter='\n')
    liste_good=[]

    liste_position = []

    for element in genome_parser:
        #Avoid repetition from couple breakpoint
        if 'left' in str(element.description):
            #positon changes if it is a back up insertion
            if element.description.split('_')[2] == 'backup':
                pos = int(element.description.split('_')[4])
            else:
                pos = int(element.description.split('_')[3])
            test_in = input_bed[(input_bed['chr'] == element.description.split('_')[1]) & (input_bed['position'] > pos-100) & (input_bed['position'] <= pos-31)]
            liste_in=test_in.in_degree.tolist()
            liste_out=test_in.out_degree.tolist()
            percentage_concatenate = (liste_in.count(1)+liste_in.count(2)+liste_out.count(1)+liste_out.count(2))/(len(liste_in)+len(liste_out))
            if percentage_concatenate > 0.50:
                liste_good.append(element.description.split('_')[0])

    print(liste_good)
    genome_parser = SeqIO.parse(open(bkpt), "fasta")
    for element in genome_parser :
        print(element)
        if element.description.split('_')[0] in liste_good :
            outp_find.writerow([">"+element.description,element.seq])

    #break
#print ("HEY",dico_FP)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# -*- coding: utf-8 -*-

# base import
import argparse
import sys
import os

# specific import
import random
from collections import defaultdict
import re

# log import
import logging


# logger configuration
def conf_logger():
    """ Set configuraion of root logger """

    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    formatter = logging.Formatter('%(levelname)s : %(message)s')

    steam_handler = logging.StreamHandler()
    steam_handler.setLevel(logging.DEBUG)
    steam_handler.setFormatter(formatter)
    logger.addHandler(steam_handler)


# argparse control argument function
def file_exist(filename):
    """ Check if file exist """

    if not os.path.isfile(filename):
        raise argparse.ArgumentTypeError("we can't access to %s file"
                                         % filename)
    return str(filename)


def unsigned_int(numbre):
    """ Check if numbre is a unsigned integer"""

    inumber = int(numbre)
    if numbre < 0:
        raise argparse.ArgumentTypeError("%s isn't a positive int value"
                                         % numbre)
    return inumber


def file_with_extension_exist(filename):
    """ Check if file with vde or fasta doesn't exist """

    for ext in ["eva", "fasta"]:
        if os.path.isfile(str(filename)+"."+str(ext)):
            raise argparse.ArgumentTypeError(
                "we need %s.%s file not exist" % str(filename), str(ext))

    return str(filename)


# user argument check
def check_interval(first, second):
    """ If intervale isn't valid generate a warning and
     return the good interval"""

    if first > second:
        logging.getLogger().warning(
            "[%d, %d] isn't a valid interval we use [%d, %d]"
            % (first, second, second, first))
        return second, first

    if first == second:
        logging.getLogger().warning(
            "[%d, %d] interval content just one number"
            % (first, second))
        return first, second

    return first, second


# specific function
# def pos_near(base_list, pos, min_dist):
#     """ Check if pos isn't near a pos in base_list """
#     for base_pos in base_list:
#         if abs(base_pos - pos) < min_dist:
#             return True

#     return False


def generate_snp_del(seq, pos_del, pos_snp, del_size):
    """ Create a SNP and a deletion """

    nuc = ['A', 'C', 'T', 'G']

    nuc.remove(seq[pos_snp])

    seq = seq[:pos_del] + seq[pos_del+del_size:]

    seq = seq[:pos_snp] + nuc[random.randint(0, 2)] + seq[pos_snp+1:]

    return seq


def write_vde(file_handler, pos, type, comment):
    """ Write information in vde format """

    file_handler.write("%s,%s,%s\n" % (pos, type, comment))


def write_seq(file_handler, comment, seq):
    """ Write information in fasta format """
    file_handler.write(">%s\n" % comment)
    file_handler.write("%s\n" % seq)


def snp_position(placement, dist_min, dist_max, del_pos):
    """ Compute the position of snp """
    if(placement == "b"):
        return del_pos - random.randint(dist_min, dist_max)
    elif(placement == "a"):
        return del_pos + random.randint(dist_min, dist_max)
    elif(placement == "r"):
        if random.random() > 0.5:
            return del_pos - random.randint(dist_min, dist_max)
        else:
            return del_pos + random.randint(dist_min, dist_max)
    else:
        logging.getLogger().warning("%s placement option isn't valid we place snp before deletion !" % placement)
        return del_pos - random.randint(dist_min, dist_max)

# Main programme function
def main():
    """ The main function of make_snp_deletions no argument """

    logger = logging.getLogger()
    parser = argparse.ArgumentParser(
        prog="make_snp_deletions",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("-g", "--genome", type=file_exist,
                        help="fasta file content the genome", required=True)
    parser.add_argument("-o", "--output", type=file_with_extension_exist,
                        help="""prefix of output file .fasta is genome with SNP
                         and deletion and .vde is the postion and type for each
                          'variant""", required=True)
    parser.add_argument("-n", "--number-del", type=unsigned_int,
                        help="number of deletions to generate", default=1)
    parser.add_argument("-m", "--min-size-del", type=unsigned_int,
                        help="minimal size of the deletions (in bp)",
                        default=100)
    parser.add_argument("-M", "--max-size-del", type=unsigned_int,
                        help="maximal size of the deletions (in bp)",
                        default=150)
    parser.add_argument("-s", "--min-dist-snp", type=unsigned_int,
                        help="minimal distance between snp and deletion (in bp)",
                        default=5)
    parser.add_argument("-S", "--max-dist-snp", type=unsigned_int,
                        help="maximal distance between snp and deletion (in bp)",
                        default=31)
    parser.add_argument("-d", "--variant-dist", type=unsigned_int,
                        help="distance minimal between two variant (in bp)",
                        default=232)
    parser.add_argument("-p", "--placement", type=str,
                        help="placement of snp compared to deletion",
                        default="b", choices=("b", "a", "r"))

    # Parse cli argument
    arg = vars(parser.parse_args())

    # Check interval
    del_size_min, del_size_max = check_interval(
        arg["min_size_del"], arg["max_size_del"])
    dist_snp_min, dist_snp_max = check_interval(
        arg["min_dist_snp"], arg["max_dist_snp"])

    # Check variant distance
    if arg["variant_dist"] <= (arg["max_size_del"] + arg["max_dist_snp"]):
        logger().warning(
            "variant distance is minus possible variant max size.")

    comment = ""
    comment2seq = defaultdict(str)
    genome_size = 0
    with open(arg["genome"]) as genome_file:
        for line in genome_file:
            if line.startswith(">"):
                comment = re.split("\s", line.lstrip(">").rstrip())[0]
            else:
                genome_size += len(line.rstrip())
                comment2seq[comment] += line.rstrip()

    nuc_per_del = genome_size / arg["number_del"]

    # output file openning
    vde_file = open(arg["output"]+".eva", "a")
    output_file = open(arg["output"]+".fasta", "a")

    # generate snp and deletion
    seq_del_cpt = 0
    del_cpt = 0
    list_pos = list()
    for comment in comment2seq.keys():
        del_pos = 0
        while seq_del_cpt < (len(comment2seq[comment]) / nuc_per_del):

            del_cpt += 1
            seq_del_cpt += 1

            del_pos_max = del_pos + arg["variant_dist"] * 2
            del_pos = random.randint(del_pos+arg["variant_dist"], del_pos_max)
            del_size = random.randint(del_size_min, del_size_max);
            snp_pos = snp_position(arg["placement"], dist_snp_min, dist_snp_max, del_pos)

            if (del_pos + del_size) > len(comment2seq[comment]):
                logger.warning("""We can't create another deletion in this 
                sequence we create %d deletion""" % seq_del_cpt)
                break
            
            comment2seq[comment] = generate_snp_del(
                comment2seq[comment], del_pos, snp_pos, del_size)
            
            write_vde(vde_file, snp_pos, "multi_snp", comment)
            write_vde(vde_file, del_pos, "homo", comment)

        seq_del_cpt = 0
        write_seq(output_file, comment, comment2seq[comment])

    # output file closeing
    vde_file.close()
    output_file.close()


if __name__ == "__main__":
    conf_logger()
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''Program to generate random deletions in a given genome. C. Lemaitre'''


# 25/07/2013 modif pour avoir un format bed pour les positions de la délétion
import getopt, sys
import random
        

def usage():
    '''Usage'''
    print "-----------------------------------------------------------------------------"
    print sys.argv[0]," : deletion simulator"
    print "-----------------------------------------------------------------------------"
    print "usage: ",sys.argv[0]," -g genome_file -o output [-n nb_del] [-m min_length] [-M max_length] [-s separator] [-N] [-b]"
    print "  -g: input fasta file containing the genome sequence(s)"
    print "  -o: file preffix for output files (.fasta : the new genome, .del.fasta : the deleted sequences, .del.txt : the positions of deletions)"
    print "  -n: number of deletions to generate, default=1"
    print "  -m: min size of the deletions (in bp), default = 100"
    print "  -M: max size of the deletions (in bp), default = 500"
    print "  -s: min distance between two consecutive deletions (in bp), default = 1, only positive separator are taken into account. For now it can not generate overlapping deletions."
    print "  -N: autorize N inside the deletion (but still not in the borders)"
    print "  -b: bed format for output of the positions of deletions (instead of .del.txt will be .del.bed)"
    print "  -h: help"
    print "-----------------------------------------------------------------------------"
    sys.exit(2)


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hg:n:o:m:M:s:Nb", ["help", "genome=", "num=", "output=", "min=", "max=", "sep=", "enableN", "bed"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err)  # will print something like "option -a not recognized"
        usage()
        sys.exit(2)

    # Default parameters
    sep=1
    min_length=100
    max_length=500
    genome_file=0
    nb_del=1
    output=0
    enableN=0
    bed_format=0
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
        elif opt in ("-o", "--output"):
            output = arg
        elif opt in ("-g", "--genome"):
            genome_file = arg
        elif opt in ("-n", "--num"):
            nb_del = int(arg)
        elif opt in ("-m", "--min"):
            min_length = int(arg)
        elif opt in ("-M", "--max"):
            max_length = int(arg)
        elif opt in ("-s", "--sep"):
            sep = int(arg)
            if sep<0:
                sep=0
        elif opt in ("-N", "--enableN"):
            enableN=1
        elif opt in ("-b", "--bed"):
            bed_format=1
        else:
            assert False, "unhandled option"

    if genome_file == 0 or output==0:
        print "Missing arguments"
        usage()
        sys.exit(2)
    elif min_length<=0 or max_length<min_length:
        print "Error in parameters : deletion length must respect the following condition : 0 < min_length <= max_length"
        sys.exit(2)
    elif nb_del<=0:
        print "Error in parameters : number of deletions should be greater than 0"
        sys.exit(2)
    else:
        
        # defining special characters :
        char_seq="$"
        char_begin="b"
        char_end="e"
        char_N="N"
        char_set="1234567890"+char_seq+char_begin+char_end;
        if enableN==0:
            char_set=char_set+char_N
                
        
        # Getting the genome sequence
        cumul_seq=""
        total_length=0
        names=[]
        nb_seq=0
        nb_col_fasta=0
        filin=open(genome_file,"r")
        for line in filin:
            if line[0] ==">":
                # header
                current_name=(line.lstrip(">")).rstrip("\n")
                names.append(current_name)
                if nb_seq>0:
                    cumul_seq+="$"
                nb_seq+=1
            else:
                current_seq=(line.rstrip("\n")).upper()
                le=len(current_seq)
                cumul_seq+=current_seq
                total_length+=le
                # we record the fasta format (width) to write output in the same format
                if le > nb_col_fasta:
                    nb_col_fasta=le
        filin.close()
        cumul_seq+="$"
        total_length+=nb_seq #  to account for the $


#        # To test :
#        test_pos=[25,5,26]
#        test_length=[5,12,20]
#        remaining_length=total_length
#        new_seq=cumul_seq
#        for i in range(len(test_pos)):
#            left_pos=test_pos[i]
#            del_length=test_length[i]


        # Placing the deletions
        nb_ok=0  # nb of placed deletions
        nb_boucle=0  # to limit the running time if deletions are too difficult to place
        remaining_length=total_length
        new_seq=cumul_seq
        while nb_ok < nb_del and nb_boucle < (20*nb_del):
            left_pos=random.randint(1,remaining_length)
            del_length=random.randint(min_length,max_length)            
            right_pos=left_pos+del_length
            # checking if deletion is possible
            # extend the region from both sides
            if right_pos+sep<remaining_length and left_pos-sep>=0:
                del_seq_ext=new_seq[(left_pos-sep):(right_pos+sep)]
                borders=new_seq[(left_pos-sep):(left_pos+sep)]+new_seq[(right_pos-sep):(right_pos+sep)]
                if not containsAny(del_seq_ext,char_set) and not char_N in borders:
                    # good deletion : does not contain any end of sequence (chr) or begin or end of previous deletions
                    # and near the borders does not have any N
                    nb_ok+=1
                    # inserting the deletion : replacing the delerted sequence by b125e (for an del of length 125)
                    tmp_seq=new_seq[0:left_pos]+char_begin+str(del_length)+char_end+new_seq[right_pos:]
                    new_seq=tmp_seq
                    remaining_length+=2+len(str(del_length))-del_length# new_seq length
            nb_boucle+=1
                
        if nb_ok < nb_del:
            print "Warning: too difficult to place ",str(nb_del)," deletions, only ",str(nb_ok)," placed"

#        print "placed deletions : "+str(nb_ok)


        # Printing the output
        filout_fasta=open(output+".fasta","w")
        filout_del=open(output+".del.fasta","w")
        if bed_format==1:
            filout_pos=open(output+".del.bed","w")
        else:
            filout_pos=open(output+".del.txt","w")

        # Reading the cumul sequence to get the positions and write the 2 fasta files
        # Note : positions are 0-based and begin and end positions are included in the deletion : [5-10] length = 11
        index_seq=0
        chr_seq=""
        old_pos=0 # position in initial sequence (without deletion)
        new_pos=0 # position in new sequence (with the deletions)
        del_begin=0
        inside_del=False
        compt_del=0
        current_name=names[index_seq]
        header=current_name
        del_length_str=""

        # position file header for txt format
        if bed_format==0:
            filout_pos.write("id\tname\tpos\tlength\tinit.inf\tinit.sup\n")
        for c in new_seq:
            if c==char_seq:
                # writing previous sequence :
                fasta_lines=writeFasta(chr_seq,header,nb_col_fasta)
                filout_fasta.write(fasta_lines)
                index_seq+=1
                if index_seq<nb_seq:  # if not the last $
                    current_name=names[index_seq]
                    header=current_name
                    cumul_seq=cumul_seq[old_pos+1:]
                    old_pos=0
                    new_pos=0
                    chr_seq=""
            elif c==char_begin:
                del_begin=old_pos
                del_length_str=""
            elif c==char_end:
                compt_del+=1
                del_length=int(del_length_str)
                del_end=del_begin+del_length
                # Getting the sequence to delete using the coordinates on the initial sequence
                del_seq=cumul_seq[del_begin:del_end]
                old_pos+=del_length
                # Writing the deleted sequence :
                del_header="deletion_"+str(compt_del)+" : "+current_name+"_"+str(new_pos)
                fasta_lines=writeFasta(del_seq,del_header,nb_col_fasta)
                filout_del.write(fasta_lines)
                # Writing the deletion positions in text files
                if bed_format==1:
                    ## ordre des champs : ch_name, pos_begin, pos_end, id, length, init.begin, init.end
                    filout_pos.write(current_name+"\t"+str(new_pos)+"\t"+str(new_pos+1)+"\t"+str(compt_del)+"\t"+str(del_end-del_begin)+"\t"+str(del_begin)+"\t"+str(del_end)+"\n")
                else:
                    filout_pos.write(str(compt_del)+"\t"+current_name+"\t"+str(new_pos)+"\t"+str(del_end-del_begin)+"\t"+str(del_begin)+"\t"+str(del_end)+"\n")
            elif c in "0123456789":
                del_length_str+=c
            else:
                chr_seq+=c
                old_pos+=1
                new_pos+=1

        filout_fasta.close()
        filout_del.close()
        filout_pos.close()

#        print "recovered and printed deletions : "+str(compt_del)

def containsAny(str, set):
    """Check whether 'str' contains ANY of the chars in 'set'"""
    return 1 in [c in str for c in set]

def writeFasta(sequence,name,ncol):
    fasta_lines=">"+name+"\n"
    if ncol>0:
        le=len(sequence)
        ind=0
        while ind<le:
            fasta_lines+=sequence[ind:(ind+ncol)]+"\n"
            ind+=ncol
    else:
        fasta_lines+=sequence+"\n"
    return fasta_lines
    

if __name__ == "__main__":
    main()

# exemple :
# ./make_deletions.py -g essai.fasta -n 1 -m 10 -M 12 -o output




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Program to generate reads from a fasta file (for tests). P. Marijon based one
get_subsequence.py write by C. Lemaitre.
'''

import getopt, sys
import random
        

def usage():
    '''Usage'''
    print "-----------------------------------------------------------------------------"
    print sys.argv[0]," : sub-sequence"
    print "-----------------------------------------------------------------------------"
    print "usage: ",sys.argv[0]," -f fasta_file -n numbre -l length"
    print "  -f: input fasta file"
    print "  -n: numbre of read you want(def : 1)"
    print "  -l: length of sub-sequence (def : 1)"
    print "  -h: help"
    print "-----------------------------------------------------------------------------"
    sys.exit(2)


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:n:l:", ["help", "fasta=", "num=", "len="])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err)  # will print something like "option -a not recognized"
        usage()
        sys.exit(2)

    # Default parameters
    read_len=1
    fasta_file=0
    num_loop=1
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
        elif opt in ("-f", "--fasta"):
            fasta_file = arg
        elif opt in ("-l", "--len"):
            read_len = int(arg)
        elif opt in ("-n", "--num"):
            num_loop = int(arg)
        else:
            assert False, "unhandled option"

    if fasta_file == 0 :
        print "Missing arguments"
        usage()
        return 2
    
    else:
        header = "base_header"
        sequence = ""
        filin = open(fasta_file,"r")

        for line in filin:
            if line[0] ==">":
                # header
                header = (line.lstrip(">")).rstrip("\n").rstrip(" ")
            else:
                sequence+=(line.rstrip("\n")).upper()

        filin.close()

        sequence_len = len(sequence)
        
        if sequence_len == 0 :
            print "warning we didn't find fasta sequence in file."
            return 1

        if sequence_len < read_len :
            print "warning read length is upper than sequence length we can't generate read."
            return 1
        
        for i in range(num_loop) :
            pos = sequence_len
            while pos + read_len > sequence_len :
                pos = random.randint(0, sequence_len)

            print ">"+header+"_read"+str(i)+"_pos_"+str(pos)+":"+str(pos+read_len)
            print sequence[pos:pos+read_len]

    return 0
            
if __name__ == "__main__":
    exit(main())
    
# exemple :
# ./get_subsequence -f random_1Mb.fa -b 44444

